/**
 * @file artis/kernel/AbstractModel.hpp
 * @author See the AUTHORS file
 */

/*
 * Copyright (C) 2012-2016 ULCO http://www.univ-littoral.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARTIS_KERNEL_ABSTRACT_MODEL_HPP
#define __ARTIS_KERNEL_ABSTRACT_MODEL_HPP

#include <artis/kernel/Externals.hpp>
#include <artis/kernel/Node.hpp>
#include <artis/kernel/State.hpp>
#include <artis/utils/DoubleTime.hpp>

#include <vector>

namespace artis { namespace kernel {

template < typename U, typename V >
class AbstractModel : public Node < U >
{
public:
    AbstractModel() : last_time(-1)
    { }

    virtual ~AbstractModel()
    { }

    virtual bool check(typename U::type /* t */) const
    { return true; }

    virtual void operator()(typename U::type t)
    {
        if (check(t) and (last_time != t or
                          (last_time == t and is_updated()))) {
            compute(t, last_time == t);
            last_time = t;
            stable();
        }
    }

    virtual void compute(typename U::type t, bool update) = 0;

    virtual void init(typename U::type t, const V& parameters) = 0;

    virtual bool is_computed(typename U::type t) const
    { return last_time == t; }

    virtual bool is_stable(typename U::type t) const = 0;

    virtual bool is_computed(typename U::type t, unsigned int index) const = 0;

    virtual bool is_updated() const = 0;

    virtual void restore(const State < U >& state) = 0;

    virtual void save(State < U >& state) const = 0;

    virtual void stable() = 0;

protected:
    typename U::type last_time;
//    std::vector < double > externalDatesV;
};

} }

#endif
