/**
 * @file artis/kernel/Internals.hpp
 * @author See the AUTHORS file
 */

/*
 * Copyright (C) 2012-2016 ULCO http://www.univ-littoral.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __ARTIS_KERNEL_INTERNALS_HPP
#define __ARTIS_KERNEL_INTERNALS_HPP

#include <artis/kernel/State.hpp>
#include <vector>

namespace artis { namespace kernel {

template < typename T, typename U >
class Internals
{
public:
    Internals()
    { }

    virtual ~Internals()
    { }

    bool T::* getB(unsigned int index) const
    { return internalsB.at(index); }

    int T::* getI(unsigned int index) const
    { return internalsI.at(index); }

    double T::* get(unsigned int index) const
    { return internalsD.at(index); }

    void I(std::initializer_list < std::pair < unsigned int,
           bool T::* > > internals)
    {
        for (typename std::initializer_list < std::pair < unsigned int,
                 bool T::* > >::iterator it =
                 internals.begin(); it != internals.end(); ++it) {
            if (internalsB.size() <= it->first) {
                internalsB.resize(it->first + 1);
            }
            internalsB[it->first] = it->second;
        }
    }

    void I(std::initializer_list < std::pair < unsigned int,
           int T::* > > internals)
    {
        for (typename std::initializer_list < std::pair < unsigned int,
                 int T::* > >::iterator it =
                 internals.begin(); it != internals.end(); ++it) {
            if (internalsI.size() <= it->first) {
                internalsI.resize(it->first + 1);
            }
            internalsI[it->first] = it->second;
        }
    }

    void I(std::initializer_list < std::pair < unsigned int,
           double T::* > > internals)
    {
        for (typename std::initializer_list < std::pair < unsigned int,
                 double T::* > >::iterator it =
                 internals.begin(); it != internals.end(); ++it) {
            if (internalsD.size() <= it->first) {
                internalsD.resize(it->first + 1);
            }
            internalsD[it->first] = it->second;
        }
    }

    void internalB(unsigned int index, bool T::* var)
    {
        if (internalsB.size() <= index) {
            internalsB.resize(index + 1);
        }
        internalsB[index] = var;
    }

    void internalI(unsigned int index, int T::* var)
    {
        if (internalsI.size() <= index) {
            internalsI.resize(index + 1);
        }
        internalsI[index] = var;
    }

    void internal(unsigned int index, double T::* var)
    {
        if (internalsD.size() <= index) {
            internalsD.resize(index + 1);
        }
        internalsD[index] = var;
    }

    virtual void restore(const State < U >& state)
    {
        unsigned int index = 0;

        for (typename std::vector < bool T::* >::iterator it =
                 internalsB.begin(); it != internalsB.end(); ++it) {
            state.get_internal(index, *it);
            ++index;
        }
        index = 0;
        for (typename std::vector < int T::* >::iterator it =
                 internalsI.begin(); it != internalsI.end(); ++it) {
            state.get_internal(index, *it);
            ++index;
        }
        index = 0;
        for (typename std::vector < double T::* >::iterator it =
                 internalsD.begin(); it != internalsD.end(); ++it) {
            state.get_internal(index, *it);
            ++index;
        }
    }

    virtual void save(State < U >& state) const
    {
        unsigned int index = 0;

        for (typename std::vector < bool T::* >::const_iterator it =
                 internalsB.begin(); it != internalsB.end(); ++it) {
            state.add_internal(index, *it);
            ++index;
        }
        index = 0;
        for (typename std::vector < int T::* >::const_iterator it =
                 internalsI.begin(); it != internalsI.end(); ++it) {
            state.add_internal(index, *it);
            ++index;
        }
        index = 0;
        for (typename std::vector < double T::* >::const_iterator it =
                 internalsD.begin(); it != internalsD.end(); ++it) {
            state.add_internal(index, *it);
            ++index;
        }
    }

private:
    std::vector < bool T::* > internalsB;
    std::vector < int T::* > internalsI;
    std::vector < double T::* > internalsD;
};

} } // namespace artis kernel

#endif
